import os
import click
from .api import MyelindlApi, MyelindlApiError


@click.command()
@click.option('--address', required=True, help="server address")
@click.option('--username', required=False, help="username")
@click.option('--data-port', required=False, default=5000, help="data transfer port")
def login(address, username, data_port):
    try:
        if username is None:
            username = click.prompt('Please enter username')
        password = click.prompt('Please enter password', hide_input=True)
        api = MyelindlApi(address, username, data_port)
        api.login(password)
        minio_key = api.minio_api_key()
        address = api.address
        host = address.split(':')[0]
        os.system("mc config host add bk http://{}:{} {} {} --api s3v4".format(
                  host, api.data_port, username, minio_key['data']))

        click.echo('Login success')
        server_ver = api.version()
        click.echo("Server version: {}".format(server_ver))
    except MyelindlApiError as e:
        click.echo('Fail due to {}'.format(e))
        raise



from tensorflow.keras.callbacks import Callback
from mlsteam.track import Track


class MLSteamCallback(Callback):
    def __init__(self, track: Track):
        super().__init__()
        self._track = track
        # self._track.tags_set(['keras'])
    
    def _log_metrics(self, logs, category: str, trigger: str):
        if not logs:
            return

        namespace = f"{category}/{trigger}"
        for metric, value in logs.items():
            try:
                if metric in ('batch', 'size') or metric.startswith('val_'):
                    continue
                self._track[f"{namespace}/{metric}"].log(value)
            except Exception:
                pass

    def on_train_batch_end(self, batch, logs=None):  # pylint:disable=unused-argument
        self._log_metrics(logs, 'train', 'batch')

    def on_epoch_end(self, epoch, logs=None):  # pylint:disable=unused-argument
        self._log_metrics(logs, 'train', 'epoch')

    def on_test_batch_end(self, batch, logs=None):  # pylint:disable=unused-argument
        self._log_metrics(logs, 'test', 'batch')

    def on_test_end(self, logs=None):
        self._log_metrics(logs, 'test', 'epoch')



import json
import click
from .api import MyelindlApi, MyelindlApiError

@click.command()
@click.argument('id', required=True)
def download(_id):
    try:
        api = MyelindlApi()
        result = api.checkpoint_download(_id)
        click.echo(result)
    except MyelindlApiError as e:
        click.echo("download checkpoint failed, {}".format(e))
        raise


@click.command('list')
def do_list():
    try:
        api = MyelindlApi()
        result = api.checkpoint_list()
        click.echo(json.dumps(result, indent=2, sort_keys=True))
    except MyelindlApiError as e:
        click.echo("list checkpoint failed, {}".format(e))
        raise


@click.command()
@click.argument('id', required=True)
def delete(_id):
    try:
        api = MyelindlApi()
        api.checkpoint_delete(_id)
    except MyelindlApiError as e:
        click.echo("delete a checkpoint failed, {}".format(e))
        raise


@click.command()
@click.argument('id', required=True)
def info(_id):
    try:
        api = MyelindlApi()
        result=api.checkpoint_info(_id)
        click.echo(json.dumps(result, indent=2, sort_keys=True))
    except MyelindlApiError as e:
        click.echo("show checkpoint info failed, {}".format(e))
        raise


@click.group(help='Groups of commands to manage checkpoints')
def checkpoint():
    pass


checkpoint.add_command(download)
checkpoint.add_command(do_list)
checkpoint.add_command(delete)
checkpoint.add_command(info)



import click
from mlsteam import version
from .api import MyelindlApi, MyelindlApiError


@click.command()
def info():
    try:
        api = MyelindlApi()
        server_ver = api.version()
        click.echo("Version: {}".format(version.__version__))
        click.echo("Server: {}".format(api.host))
        click.echo("Server Version: {}".format(server_ver))
        click.echo("Username: {}".format(api.username))
        click.echo("Data Port: {}".format(api.data_port))
    except MyelindlApiError as e:
        click.echo('Fail due to {}'.format(e))
        raise



import json
import click
from .api import MyelindlApi, MyelindlApiError

@click.command()
@click.argument('id', required=True)
def create(_id):
    try:
        api = MyelindlApi()
        result = api.service_create(_id)
        click.echo(result)
    except MyelindlApiError as e:
        click.echo("create service failed, {}".format(e))
        raise


@click.command('list')
def do_list():
    try:
        api = MyelindlApi()
        result = api.service_list()
        click.echo(json.dumps(result, indent=2, sort_keys=True))
    except MyelindlApiError as e:
        click.echo("list service failed, {}".format(e))
        raise


@click.command()
@click.argument('id', required=True)
def delete(_id):
    try:
        api = MyelindlApi()
        api.service_delete(_id)
    except MyelindlApiError as e:
        click.echo("delete a service failed, {}".format(e))
        raise


@click.group(help='Groups of commands to manage services')
def service():
    pass


service.add_command(create)
service.add_command(do_list)
service.add_command(delete)



import os

def get_env_var(key, default=None):
    if key not in os.environ:
        return default
    return os.environ[key]

API_SERVER_ADDRESS = get_env_var('API_SERVER_ADDRESS', '140.96.29.151')
API_BASE_URL = 'http://{}/api'.format(API_SERVER_ADDRESS)
DATA_PORT = get_env_var('DATA_PORT', '5000')



import json
import os
import re
import time
from os import system

import requests
import pexpect
import pytest

from .config import API_BASE_URL, API_SERVER_ADDRESS, DATA_PORT

headers={}
base_url=API_BASE_URL
tout=100


def setup_module(module):
    """ setup any state specific to the execution of the given module."""
    child = pexpect.spawn('mlsteam login --address {} --username superuser --data-port {}'.format(API_SERVER_ADDRESS, DATA_PORT))
    child.expect ('password:')
    child.sendline ('superuser')
    child.expect(pexpect.EOF)
    out=child.before
    exp=re.findall(b"Login success", out)
    assert exp==[b'Login success']
    config_path = os.path.join(os.getenv('HOME'), '.mlsteam', 'cred')
    with open(config_path, 'r') as cred:
        data = json.load(cred)
    headers.update({'Authorization': 'Bearer {}'.format(data['access_token'])})

    system("mlsteam data mb bk/cifar10")
    system("mlsteam data cp -r /workspace/cifar10/* bk/cifar10")
    system("mlsteam data ls bk/cifar10")
    

def teardown_module(module):
    """ teardown any state that was previously setup with a setup_module
    method.
    """
#    pytest.skip()
    system("mlsteam data rb bk/cifar10") 


"""
 user account related test cases
"""
def test_user_list():
    url = '{}/{}'.format(base_url, 'users')
    data = None
    response = requests.get(url, timeout=tout, headers=headers)
    print(response.text)
    assert response.status_code == 200


def test_user_add():
    pytest.skip()

def test_user_inactive():
    pytest.skip()

def test_user_remove():
    pytest.skip()


"""
 project related test cases
"""

def test_project_create_private():
    url = '{}/{}'.format(base_url, 'projects')
    #data1 = json.dumps({ 'name':'test_api_project', 'dataset':'cifar10', 'is_member_only':True })
    #response = requests.post(url, timeout=tout, headers=headers, data=data1) #TODO pass is_member_only
    response = requests.post(url, timeout=tout, headers=headers, data={ 'name':'test_api_project', 'dataset':'cifar10' }) #TODO pass is_member_only
    print(response.text)
    assert response.status_code == 200


def test_project_create_public():
    url = '{}/{}'.format(base_url, 'projects')
    data = None
    response = requests.post(url, timeout=tout, headers=headers, data={'name':'test_api_project_public', 'dataset':'cifar10'}) #TODO pass is_member_only
    print(response.text)
    assert response.status_code == 200


def test_project_list():
    url = '{}/{}'.format(base_url, 'projects')
    data = None
    response = requests.get(url, timeout=tout, headers=headers)
    dic=response.json()
    print(dic)
    assert response.status_code == 200



def test_project_info():
    url = '{}/{}'.format(base_url, 'projects')
    data = None
    response = requests.get(url, timeout=tout, headers=headers)
    dic=response.json()

    for proj in dic:
        if proj['name'] == 'test_api_project_public' or  proj['name'] == 'test_api_project':
            url = '{}/{}/{}'.format(base_url, 'projects', proj['id'])
            data = None
            response = requests.get(url, timeout=tout, headers=headers)
            print(response.text)
            assert response.status_code == 200



def test_project_member_list():
    url = '{}/{}'.format(base_url, 'projects')
    data = None
    response = requests.get(url, timeout=tout, headers=headers)
    dic=response.json()

    for proj in dic:
        if proj['name'] == 'test_api_project_public' or  proj['name'] == 'test_api_project':
            url = '{}/{}/{}/{}'.format(base_url, 'projects', proj['id'], 'members')
            data = None
            response = requests.get(url, timeout=tout, headers=headers)
            print(response.text)
            assert response.status_code == 200



def test_project_member_add():
    url = '{}/{}'.format(base_url, 'projects')
    data = None
    response = requests.get(url, timeout=tout, headers=headers)
    dic=response.json()

    for proj in dic:
        if proj['name'] == 'test_api_project_public' or  proj['name'] == 'test_api_project':
            url = '{}/{}/{}/{}'.format(base_url, 'projects', proj['id'], 'members')
            data = None
            response = requests.post(url, timeout=tout, headers=headers, data={'username':'test1'})
            print(response.text)
            assert response.status_code == 200


def test_project_member_remove():
    url = '{}/{}'.format(base_url, 'projects')
    data = None
    response = requests.get(url, timeout=tout, headers=headers)
    dic=response.json()

    for proj in dic:
        if proj['name'] == 'test_api_project_public' or  proj['name'] == 'test_api_project':
            url = '{}/{}/{}/{}/{}'.format(base_url, 'projects', proj['id'], 'members', 'test1')
            data = None
            response = requests.delete(url, timeout=tout, headers=headers)
            print(response.text)
            assert response.status_code == 200

"""
work related test cases
"""
def test_work_create():
    url = '{}/{}'.format(base_url, 'works')
    data = {'container':'myelintek/python-gpu:v5', 'num_gpu':'1', 'dataset':'cifar10', 'project':'test_api_project_public', 'port_list':'8888', 'user_args':"jupyter-notebook --ip 0.0.0.0 --allow-root --NotebookApp.token='JOB_ID'"} 
    response = requests.post(url, timeout=tout, headers=headers, data=data) 
    print(response.text)


def test_work_list():
    url = '{}/{}'.format(base_url, 'works')
    data = None
    response = requests.get(url, timeout=tout, headers=headers)
    print(response.text)

def test_work_info():
    url = '{}/{}'.format(base_url, 'works')
    data = None
    response = requests.get(url, timeout=tout, headers=headers)
    dic=response.json()

    for work in dic:
        if work['project'] == 'test_api_project_public' or  work['project'] == 'test_api_project':
            url = '{}/{}/{}'.format(base_url, 'works', work['id'])
            data = None
            response = requests.get(url, timeout=tout, headers=headers)
            print(response.text)
            assert response.status_code == 200


"""
Cleanup
"""
def test_work_delete():
    url = '{}/{}'.format(base_url, 'works')
    data = None
    response = requests.get(url, timeout=tout, headers=headers)
    dic=response.json()

    for work in dic:
        if work['project']['name'] == 'test_api_project_public' or  work['project']['name'] == 'test_api_project':
            time.sleep(10)
            url = '{}/{}/{}'.format(base_url, 'works', work['id'])
            print(work['id'])
            data = None
            response = requests.delete(url, timeout=tout, headers=headers)
            print(response.text)
            assert response.status_code == 200
            time.sleep(30)
            response = requests.get(url, timeout=tout, headers=headers)
            assert response.text == '{}\n'


def test_project_delete():
    url = '{}/{}'.format(base_url, 'projects')
    data = None
    response = requests.get(url, timeout=tout, headers=headers)
    dic=response.json()

    for proj in dic:
        if proj['name'] == 'test_api_project_public' or  proj['name'] == 'test_api_project':
            url = '{}/{}/{}'.format(base_url, 'projects', proj['id'])
            data = None
            response = requests.delete(url, timeout=tout, headers=headers)
            print(response.text)
            assert response.status_code == 200



import json
import click
from .api import MyelindlApi, MyelindlApiError

@click.command()
@click.argument('container', required=True)
@click.option('--dataset', help='dataset used for container jobs (for container jobs)')
@click.option('--num-gpu', type=click.INT, default=1, help='number of GPU (default: 1)')
@click.option('--port', help='port numbers to be exposed from container')
@click.argument('user-args', nargs=-1, type=click.Path())
def create(container, dataset, num_gpu, port, user_args):
    try:
        api = MyelindlApi()
        result = api.work_create(container, dataset, num_gpu, port, user_args)
        click.echo(result)
    except MyelindlApiError as e:
        click.echo("create work failed, {}".format(e))
        raise


@click.command('list')
def do_list():
    try:
        api = MyelindlApi()
        result = api.work_list()
        click.echo(json.dumps(result, indent=2, sort_keys=True))
    except MyelindlApiError as e:
        click.echo("list work failed, {}".format(e))
        raise


@click.command()
@click.argument('id', required=True)
def delete(_id):
    try:
        api = MyelindlApi()
        api.work_delete(_id)
    except MyelindlApiError as e:
        click.echo("delete a work failed, {}".format(e))
        raise

@click.command()
@click.argument('id', required=True)
def info(_id):
    try:
        api = MyelindlApi()
        result = api.work_info(_id)
        click.echo(json.dumps(result, indent=2, sort_keys=True))
    except MyelindlApiError as e:
        click.echo("list work failed, {}".format(e))
        raise


@click.group(help='Groups of commands to manage works')
def work():
    pass


work.add_command(create)
work.add_command(do_list)
work.add_command(delete)
work.add_command(info)



import click
from mlsteam.api import MyelindlApi, MyelindlApiError


@click.command('list')
def do_list():
    try:
        api = MyelindlApi()
        result = api.image_list()
        longest = 20
        if result:
            longest = max(len(x['tag']) for x in result)
        if longest < 20:
            longest = 20
        template = '| {:>10} | {:>%d} |'% longest
        header = template.format('id', 'tag')
        click.echo('=' * len(header))
        click.echo(header)
        click.echo('=' * len(header))
        for img in result:
            line = template.format(img['id'], img['tag'])
            click.echo(line)
        click.echo('=' * len(header))
    except MyelindlApiError as e:
        click.echo(str(e))
        raise


@click.command()
@click.option('--id', required=True, help="image id")
def delete(_id):
    try:
        api = MyelindlApi()
        result = api.image_delete(_id)
        click.echo(result)
    except MyelindlApiError as e:
        click.echo(str(e))
        raise


@click.command()
@click.option('--tag', required=True, help="image tag")
def pull(tag):
    try:
        api = MyelindlApi()
        result = api.image_pull(tag)
        click.echo(result)
    except MyelindlApiError as e:
        click.echo(str(e))
        raise


@click.group(help='Group of commands to manage container images')
def image():
    pass

image.add_command(do_list)
image.add_command(delete)
image.add_command(pull)


@click.group(help='Groups of commands to manage container')
def container():
    pass

container.add_command(image)



import base64
import click
import os
import json
import threading
try:
    import mlsteam.keras
except ImportError:
    pass
import mlsteam.stparams
from mlsteam.api import MyelindlApi
from mlsteam.consumer import ApiClient
from mlsteam import envs
from mlsteam.version import __version__
from mlsteam.track import Track
from mlsteam.exceptions import (
    MLSteamMissingApiTokenException,
    MLSteamInvalidApiTokenException,
    MLSteamMissingProjectNameException,
)


def init(project_name=None, api_token=None, debug=False, track_id=None):
    click.echo("mlsteam-client v{}".format(__version__))
    apiclient = ApiClient(api_token=api_token)
    project_uuid = project_name_lookup(apiclient, project_name)
    if track_id is None:
        track_id = os.getenv(envs.MLSTEAM_TRACK_ID)
    # Track
    if track_id:
        track_obj = apiclient.get_track(project_uuid, track_id)
    else:
        track_obj = apiclient.create_track(project_uuid)

    # stdout_path = "monitoring/stdout"
    # stderr_path = "monitoring/stderr"
    # traceback_path = "monitoring/traceback"
    background_jobs = []

    _track = Track(
        track_obj,
        project_uuid,
        apiclient,
        background_jobs,
        debug=debug
    )
    _track.start()
    return _track


def project_name_lookup(apiclient, name=None):
    if not name:
        name = os.getenv(envs.PROJECT_ENV)
        if name:
            click.echo("project name({}): {}".format(envs.PROJECT_ENV, name))
    if not name:
        raise MLSteamMissingProjectNameException()
    return apiclient.get_project(name)



import os
import uuid
import logging
import click
from .api import MyelindlApi, MyelindlApiError
from .utils import sizeof_fmt

@click.command('list')
@click.option('--offset', required=False, default=0, help='*optional start offset in model list')
@click.option('--limit', required=False, default=None, help='*optional start offset in model list')
def do_list(offset, limit):
    results = []
    try:
        api = MyelindlApi()
        results = api.model_list(
            offset=offset,
            limit=limit
        )
    except MyelindlApiError as e:
        click.echo('Fail due to {}'.format(str(e)))
        raise

    template = '| {:>30}|'
    header = template.format('model')

    click.echo('=' * len(header))
    click.echo(header)
    click.echo('=' * len(header))

    for ds in results:
        line = template.format(
            ds,
        )
        click.echo(line)
    click.echo('='* len(header))

@click.command()
@click.option('tag','--tag', required=True, help='model name (ex. user/model_name)')
@click.option('--offset', required=False, default=0, help='*optional start offset in model list')
@click.option('--limit', required=False, default=None, help='*optional start offset in model list')
def versions(tag, offset, limit):
    results = []
    try:
        api = MyelindlApi()
        results = api.model_versions(
            tag,
            offset=offset,
            limit=limit,
        )
    except MyelindlApiError as e:
        click.echo('Fail due to {}'.format(str(e)))
        raise

    longest = max(len(d['tag']) for d in results)
    if longest < 10:
        longest = 10
    template = '| {:>%d} | {:>20} | {:>10} | {:>10} | {:>10} | {:>10} |'% longest
    header = template.format('tag', 'name', 'version', 'size', 'type', 'user')


    click.echo('=' * len(header))
    click.echo(header)
    click.echo('=' * len(header))

    for ds in results:
        line = template.format(
            ds['tag'],
            ds['name'],
            ds['version'],
            sizeof_fmt(ds['size']),
            ds['type'],
            ds['username'],
        )
        click.echo(line)
    click.echo('='* len(header))



@click.command()
@click.option('tag','--tag', required=True, help='model tag (ex. user/model_name:tag)')
@click.option('--output_dir', required=False, default='', help='Speicify output directory  to pulled file')
def pull(tag, output_dir):
    try:
        api = MyelindlApi()
        api.model_pull(
            tag,
            output_dir
        )
        click.echo('Finish model pull {}' .format(tag))
    except MyelindlApiError as e:
        click.echo('Fail due to {}'.format(str(e)))
        raise



@click.command()
@click.option('tag','--tag', required=True, help='model tag (ex. user/model_name:tag)')
@click.option('--model-dir', required=True, help='target model directory')
@click.option('--description', required=False, default='', help='model description')
@click.option('--type', required=False, default='file', help='model type')
def push(tag, model_dir, description, _type):
    if not os.path.exists(model_dir):
        click.echo('model-dir: {} not exists!'.format(model_dir))

    temp_dir = 'mlt_tmp-{}'.format(str(uuid.uuid1())[:8])
    try:
        api = MyelindlApi()
        # api.data_upload(model_dir, temp_dir)
        result = api.model_push(
            tag,
            temp_dir,
            description,
            _type,
        )
        click.echo( 'Model {} pushed, {}'.format(tag, result))
    except MyelindlApiError as e:
        logging.warning("{}".format(e))
        click.echo( 'Dataset publish fail, due to {}'.format(str(e)))
        raise


@click.command()
@click.option('tag','--tag', required=True, help='model tag (ex. user/model_name:tag)')
def delete(tag):
    try:
        api = MyelindlApi()
        api.model_delete(tag)
        click.echo('Model deleted {}'.format(tag))
    except MyelindlApiError as e:
        click.echo('Fail due to {}'.format(str(e)))
        raise



@click.command()
@click.option('tag','--tag', required=True, help='model tag (ex. user/model_name:tag)')
def info(tag):
    result = {}
    try:
        api = MyelindlApi()
        result = api.model_info(tag)
    except MyelindlApiError as e:
        click.echo('Fail due to {}'.format(str(e)))
        raise

    longest = max(len(str(v)) for v in result.values())
    longest = longest if longest >= 10 else 10
    template = '| {:>16} | {:>%d} |'% longest
    header = template.format('key', 'value')

    click.echo('=' * len(header))
    click.echo(header)
    click.echo('=' * len(header))

    for k,v in result.iteritems():
        line = template.format(k, v)
        click.echo(line)
    click.echo('='* len(header))


@click.group()
def model():
    pass


model.add_command(do_list)
model.add_command(versions)
model.add_command(info)
model.add_command(push)
model.add_command(pull)
model.add_command(delete)



import os
import click
import logging
import yaml

filename = 'mlsteam.yml'
HOME = os.getenv('HOME', '/')

# PWD/mlsteam.yml
# HOME/mlsteam.yml
# HOME/lab/mlsteam.yml


def get_value(key, default=None):
    dirs = [os.getcwd(), HOME, os.path.join(HOME, 'lab')]
    param_file = ""
    for _dir in dirs:
        file_lookup = os.path.join(_dir, filename)
        if os.path.exists(file_lookup):
            param_file = file_lookup
            break

    if not param_file:
        logging.warning(
            "use default value for {}, mlsteam.yml not found.".format(key))
        return default

    params = {}
    with open(param_file, encoding='utf-8', mode='r') as f:
        params = yaml.safe_load(f)
    if params is None or "params" not in params:
        logging.warning(
            "use default value for {}, undefined variable.".format(key))
        return default
    for k, v in params["params"].iteritems():
        if key == k:
            if isinstance(params["params"][key], list):
                click.echo("hyperparameter - {}: {}".format(key,
                           params["params"][key][0]))
                return params["params"][key][0]
            click.echo(
                "hyperparameter - {}: {}".format(key, params["params"][key]))
            return params["params"][key]
    click.echo("hyperparameter - {}: {}(default)".format(key, default))
    return default



import pexpect
import re
import os
import time
from os import system

from .config import API_SERVER_ADDRESS, DATA_PORT

def setup_module(module):
    child = pexpect.spawn('mlsteam login --address {} --username superuser --data-port {}'.format(API_SERVER_ADDRESS, DATA_PORT))
    child.expect ('password:')
    child.sendline ('superuser')
    child.expect(pexpect.EOF)
    out=child.before
    exp=re.findall(b"Login success", out)
    assert exp==[b'Login success']
    system("mlsteam data mb bk/cifar10")
    system("mlsteam data cp -r /workspace/cifar10/* bk/cifar10")
    system("mlsteam data ls bk/cifar10")

def teardown_module(module):
    """ teardown any state that was previously setup with a setup_module
    method.
    """
    #clean jobs if any test case failed
    job_ids=os.popen("mlsteam job list | awk '/job1/ {print $2}'").read()
    print(job_ids)
    print(job_ids.split('\n'))
    for job_id in job_ids.split('\n'):
        if job_id=='': continue
        ret=system("mlsteam job delete --job-id "+job_id)
        assert ret==0

    #clean dataset
    system("mlsteam data rb bk/cifar10")
    system("mlsteam data rb bk/wrong_dataset")

    #clean project
    proj_id=os.popen("mlsteam project list | awk '/test_negative_project/ {print $2}'").read()
    print(proj_id)
    ret=system("mlsteam project delete --id "+proj_id)

#wrong password is covered in api negative tests

# data
def test_data_cp_wrong_bucket():
    ret=system("mlsteam data cp /workspace/tests/fake_dataset/file1 bk/wrong_bucket")
    assert ret!=0

def test_data_ls_wrong_bucket():
    ret=system("mlsteam data ls bk/wrong_bucket")
    assert ret!=0

def test_data_rm_wrong_bucket():
    ret=system("mlsteam data rm bk/wrong_bucket/file1")
    assert ret!=0

def test_data_rb_wrong_bucket():
    ret=system("mlsteam data rb wrong_bucket")
    assert ret!=0

#project

def test_project_create_wrong_dataset():
    ret=system("mlsteam project create test_negative_project wrong_dataset")
    assert ret!=0

def test_project_create_same_name():
    ret=system("mlsteam project create test_negative_project cifar10")
    ret=system("mlsteam project create test_negative_project cifar10")
    assert ret!=0

def test_project_delete_wrong_id():
    ret=system("mlsteam project delete --id wrong_id")
    assert ret!=0

#job

def test_job_submit_wrong_project():
    ret=system("PROJECT=wrong project mlsteam job submit training --job-name job1 --package-path /workspace/example/cifar10_estimator")
    assert ret!=0

def test_job_submit_wrong_package():
    ret=system("PROJECT=test_negative_project project mlsteam job submit training --job-name job1 --package-path /wrong_path")
    assert ret!=0

def test_job_delete_wrong_id():
    ret=system("mlsteam job delete --job-id wrong_id")
    assert ret!=0





import click
import atexit
import threading
import traceback
from time import sleep
from mlsteam.consumer import ApiClient, DiskCache, ConsumerThread


class Track(object):
    def __init__(self,
        track: dict,
        project_uuid: str,
        apiclient: ApiClient,
        background_jobs: list,
        debug: bool
    ):
        self._info = track
        self._project_uuid = project_uuid
        self._lock = threading.RLock()
        self._apiclient = apiclient
        self._background_jobs = background_jobs
        self._debug = debug
        self._waiting_cond = threading.Condition(lock=self._lock)
        self._cache = DiskCache(f"{project_uuid}/{track['name']}", debug=self._debug)
        self._consumer = ConsumerThread(
            self._lock,
            self._cache,
            self._apiclient,
            project_uuid,
            track['bucket_name'],
            sleep_time=1
        )

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_tb is not None:
            traceback.print_exception(exc_type, exc_val, exc_tb)
        self.stop()

    def __getitem__(self, key):
        return Handler(self, self._cache, key)

    def __setitem__(self, key, value):
        self.__getitem__(key).assign(value)

    def lock(self):
        return self._lock

    def start(self):
        atexit.register(self._shutdown_hook)
        self._consumer.start()

    def stop(self):
        if self._consumer.is_running():
            self._consumer.disable_sleep()
            self._consumer.wake_up()
            self._wait_queu_empty(self._cache)
            self._consumer.interrupt()
            click.echo("mlsteam-client stopped")
        self._consumer.join()

    def _wait_queu_empty(self, cache: "DiskCache"):
        while True:
            qsize = cache.queue_size()
            if qsize == 0:
                break
            sleep(1)

    def _shutdown_hook(self):
        self.stop()

    def tags_set(self, tags: list):
        self._apiclient.add_tags(self._project_uuid, self._info['id'], tags)


class Handler(object):
    def __init__(self, track: Track, cache: DiskCache, key: str):
        self._track = track
        self._cache = cache
        self._key = key

    def __getitem__(self, key):
        return Handler(self._track, self._cache, self._key)

    def __setitem__(self, key, value):
        self[key].assign(value)

    def assign(self, value):
        with self._track.lock():
            self._cache.assign(self._key, value)

    def log(self, value):
        with self._track.lock():
            self._cache.log(self._key, value)



import os
import uuid
import click
from .api import MyelindlApi, MyelindlApiError
from .utils import sizeof_fmt


# @click.command()
# @click.option('dataset_type', '--type', type=click.Choice(READERS.keys()), required=True)  # noqa
# @click.option('--data-dir', required=True, help='Where to locate the original data.')  # noqa
# @click.option('--output-dir', required=True, help='Where to save the transformed data.')  # noqa
# @click.option('splits', '--split', required=True,
#               multiple=True, help='The splits to transform (ie. train, test, val).')  # noqa
# @click.option('--debug', is_flag=True, help='Set level logging to DEBUG.')
# def conversion(dataset_type, data_dir,  output_dir, splits, debug):
#     from readers import get_reader, READERS
#     from writers import get_writer
#     import tensorflow as tf
#     tf.logging.set_verbosity(tf.logging.INFO)
#     if debug:
#         tf.logging.set_verbosity(tf.logging.DEBUG)

#     try:
#         reader = get_reader(dataset_type)
#     except ValueError as e:
#         tf.logging.error('Error getting reader: {}'.format(e))
#         return

#     try:
#         writer = get_writer(dataset_type)
#         for split in splits:
#             split_reader = reader(data_dir, split)
#             writer = writer(split_reader, output_dir, split)
#             writer.save()

#             tf.logging.info('Composition per class ({})'.format(split))
#     except ValueError as e:
#         tf.logging.error('Error reading dataset: {}'.format(e))


@click.command()
@click.option('name', '--name', required=True, help='Dataset name')
@click.option('--data-dir', required=True, help="Where to locate the directory contains coverted data.")
@click.option('--id', required=False, help="Specify id to update published dataset.")
@click.option('--type', required=False, help="Specify type of published dataset.(ex: object, file)")
@click.option('--description', required=False, help='Dataset description')
def publish(data_dir, name, _id, _type, description):

    if not os.path.exists(data_dir):
        click.echo('data-dir: {} not exists!'.format(data_dir))

    temp_dir = 'mlt_tmp-{}'.format(str(uuid.uuid1())[:8])
    try:
        api = MyelindlApi()
        # api.data_upload(data_dir, temp_dir)

        dataset_id = _id if _id else ''
        dataset_type = _type if _type else 'file'

        result = api.dataset_publish(
            _id=dataset_id,
            _type=dataset_type,
            name=name,
            description=description,
            data_dir=temp_dir,
        )

        click.echo('Dataset {} published with id {}'.format(name, result['id']))
    except MyelindlApiError as e:
        click.echo('Dataset publish fail, due to {}'.format(str(e)))
        raise


@click.command()
@click.option('id', '--id', required=True, help="Which dataset you want to unpublish")
def unpublish(_id):
    api = MyelindlApi()
    try:
        api.dataset_unpublish(_id)
        click.echo('Dataset {} unpublished'.format(_id))
    except MyelindlApiError as e :
        click.echo('Fail due to {}'.format(str(e)))
        raise


@click.command('list')
def do_list():
    api = MyelindlApi()
    try:
        result = api.dataset_list()
    except MyelindlApiError as e :
        click.echo('Fail due to {}'.format(str(e)))
        raise
    longest = 10
    if result['datasets']:
        longest = max(len(d['name']) for d in result['datasets'])
    if longest < 10:
        longest = 10
    template = '| {:>10} | {:>%d} | {:>30} | {:>10} | {:>10} | {:>10} |'% longest
    header = template.format('id', 'name', 'description','size', 'type', 'user')

    click.echo('=' * len(header))
    click.echo(header)
    click.echo('=' * len(header))

    for ds in result['datasets']:
        line = template.format(
            ds['id'],
            ds['name'],
            ds['description'],
            sizeof_fmt(ds['size']),
            ds['type'],
            ds['username'],
        )
        click.echo(line)
    click.echo('='* len(header))


@click.command()
@click.option('id', '--id', required=True, help="Which dataset you want to read information")
def info(_id):
    api = MyelindlApi()
    try:
        result = api.dataset_info(_id)
    except MyelindlApiError as e :
        click.echo('Fail due to {}'.format(str(e)))
        raise

    template = '| {:>20} | {:>30}|'
    header = template.format('Field', 'Value')

    click.echo('=' * len(header))
    click.echo(header)
    click.echo('=' * len(header))

    for k, v in result.iteritems():
        line = template.format(k, v)
        click.echo(line)
    click.echo('='* len(header))


@click.command()
@click.option('--id',required=True)
@click.option('dir','--dir',required=False)
def browse(_id, _dir):

    api = MyelindlApi()
    try:
        result = api.dataset_items(_id, _dir)
        for file in result:
            if file['type'] == 'dir':
                click.echo('{}/'.format(file['basename']))
            else:
                click.echo(file['basename'])
    except Exception as e:
        click.echo("Fail due to {}".format(str(e)))
        raise
@click.group(help='Groups of commands to manage datasets')
def dataset():
    pass


#dataset.add_command(conversion)
#dataset.add_command(publish)
#dataset.add_command(unpublish)
dataset.add_command(do_list)
#dataset.add_command(browse)
#dataset.add_command(info)



import requests                                                                          
import pexpect                                                                           
import pytest                                                                            
import json                                                                              
import os                                                                                
import re                                                                                
import time                                                                              

from os import system                                                                    

from .config import API_BASE_URL, API_SERVER_ADDRESS, DATA_PORT


headers={}                                                                               
base_url=API_BASE_URL                                        
tout=100                                                                                 
                                                                                         
                                                                                         
def setup_module(module):                                                                
    """ setup any state specific to the execution of the given module."""                
    child = pexpect.spawn('mlsteam login --address {} --username superuser --data-port {}'.format(API_SERVER_ADDRESS, DATA_PORT))
    child.expect ('password:')                                                           
    child.sendline ('superuser')                                                         
    child.expect(pexpect.EOF)                                                            
    out=child.before                                                                     
    exp=re.findall(b"Login success", out)                                                
    assert exp==[b'Login success']                                                       
    config_path = os.path.join(os.getenv('HOME'), '.mlsteam', 'cred')                    
    with open(config_path, 'r') as cred:                                                 
        data = json.load(cred)                                                           
    headers.update({'Authorization': 'Bearer {}'.format(data['access_token'])})          
                                                                                         
    system("mlsteam data mb bk/cifar10")                                                 
    system("mlsteam data cp -r /workspace/cifar10/* bk/cifar10")                         
    system("mlsteam data ls bk/cifar10")                                                 
                                                                                         
                                                                                         
def teardown_module(module):                                                             
    """ teardown any state that was previously setup with a setup_module                 
    method.                                                                              
    """                                                                                                                                                
    system("mlsteam data rb bk/cifar10")

    url = '{}/{}'.format(base_url, 'projects')
    data = None
    response = requests.get(url, timeout=tout, headers=headers)
    dic=response.json()

    for proj in dic:
        if proj['name'] == 'test_api_project_negative':
            url = '{}/{}/{}'.format(base_url, 'projects', proj['id'])
            data = None
            response = requests.delete(url, timeout=tout, headers=headers)
            print(response.text)
            assert response.status_code == 200
                                                                                         
                                                                                         
"""                                                                                      
 user account related test cases                                                         
"""  
def test_wrong_password():
    child = pexpect.spawn('mlsteam login --address {} --username superuser'.format(API_SERVER_ADDRESS))  
    child.expect ('password:')                                                           
    child.sendline ('wrong_password')                                                         
    child.expect(pexpect.EOF)                                                            
    out=child.before                                                                     
    exp=re.findall(b"Login success", out)                                                
    assert exp != [b'Login success']  

"""
 project related test cases
"""
def test_project_create_no_dataset():
    url = '{}/{}'.format(base_url, 'projects')
    response = requests.post(url, timeout=tout, headers=headers, data={ 'name':'test_api_project_negative', 'dataset':'wrong_dataset' }) #TODO pass is_member_only
    print(response.text)
    assert response.status_code != 200


def test_project_create_exists():
    url = '{}/{}'.format(base_url, 'projects')
    response = requests.post(url, timeout=tout, headers=headers, data={ 'name':'test_api_project_negative', 'dataset':'cifar10' }) #TODO pass is_member_only
    response = requests.post(url, timeout=tout, headers=headers, data={ 'name':'test_api_project_negative', 'dataset':'cifar10' }) #TODO pass is_member_only
    print(response.text)
    assert response.status_code != 200

def test_project_member_add_wrong_name():
    url = '{}/{}'.format(base_url, 'projects')
    data = None
    response = requests.get(url, timeout=tout, headers=headers)
    dic=response.json()

    for proj in dic:
        if proj['name'] == 'test_api_project_negative':
            url = '{}/{}/{}/{}'.format(base_url, 'projects', proj['id'], 'members')
            data = None
            response = requests.post(url, timeout=tout, headers=headers, data={'username':'wrong_username'})
            print(response.text)
            assert response.status_code != 200

def test_project_member_remove_owner():
    url = '{}/{}'.format(base_url, 'projects')
    data = None
    response = requests.get(url, timeout=tout, headers=headers)
    dic=response.json()

    for proj in dic:
        if proj['name'] == 'test_api_project_negative':
            url = '{}/{}/{}/{}/{}'.format(base_url, 'projects', proj['id'], 'members', 'superuser')
            data = None
            response = requests.delete(url, timeout=tout, headers=headers)
            print(response.text)
            assert response.status_code != 200


"""
work related test cases
"""
def test_work_create_wrong_project():
    url = '{}/{}'.format(base_url, 'works')
    data = {'container':'myelintek/python-gpu:v5', 'num_gpu':'1', 'dataset':'cifar10', 'project':'wrong_project', 'port_list':'8888', 'user_args':"jupyter-notebook --ip 0.0.0.0 --allow-root --NotebookApp.token='JOB_ID'"}
    response = requests.post(url, timeout=tout, headers=headers, data=data)
    print(response.text)
    assert response.status_code != 200

def test_work_create_wrong_dataset():
    url = '{}/{}'.format(base_url, 'works')
    data = {'container':'myelintek/python-gpu:v5', 'num_gpu':'1', 'dataset':'wrong_dataset', 'project':'test_api_project_negative', 'port_list':'8888', 'user_args':"jupyter-notebook --ip 0.0.0.0 --allow-root --NotebookApp.token='JOB_ID'"}
    response = requests.post(url, timeout=tout, headers=headers, data=data)
    print(response.text)
    assert response.status_code != 200

def test_work_create_wrong_container():
    url = '{}/{}'.format(base_url, 'works')
    data = {'container':'wrong_container', 'num_gpu':'1', 'dataset':'cifar10', 'project':'test_api_project_negative', 'port_list':'8888', 'user_args':"jupyter-notebook --ip 0.0.0.0 --allow-root --NotebookApp.token='JOB_ID'"}
    response = requests.post(url, timeout=tout, headers=headers, data=data)
    print(response.text)
    assert response.status_code != 200



import os


def data_upload(host, port, username, keyfile, local_dir, remote_dir):
    if local_dir.endswith(os.sep):
        local_dir = local_dir[:-1]
    os.system("rsync -az --delete --rsh='ssh -p {} -i {} -o ForwardX11=no' {}/ {}@{}:~/{}".format(
        port, keyfile, local_dir, username, host, remote_dir,
    ))


#def ftp_upload_dir(host, username, password, local_dir, remote_dir):
#    sf = ftputil.session.session_factory(
#        base_class=ftplib.FTP,
#        port=21,
#        use_passive_mode=False
#    )
#    local_dir = local_dir.strip(os.sep)
#    with ftputil.FTPHost(host, username, password, session_factory=sf) as ftp:
#        for base, dirs, files in os.walk(local_dir):
#            remote_base = base.replace(local_dir, remote_dir)
#
#            if not ftp.path.exists(remote_base):
#                ftp.mkdir(remote_base)
#
#            for f in files:
#                local_f = os.path.join(base, f)
#                remote_f = ftp.path.join(remote_base, f)
#                ftp.upload(local_f, remote_f)
#

def sizeof_fmt(num, suffix='B'):
    for unit in ['','K','M','G','T','P','E','Z']:
        if abs(num) < 1024.0:
            return "%3.1f%s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f%s%s" % (num, 'Yi', suffix)



import re
import os
import time
from os import system

import pexpect

from .config import API_SERVER_ADDRESS, DATA_PORT

def setup_module(module):
    child = pexpect.spawn('mlsteam login --address {} --username superuser --data-port {}'.format(API_SERVER_ADDRESS, DATA_PORT))
    child.expect ('password:')
    child.sendline ('superuser')
    child.expect(pexpect.EOF)
    out=child.before
    exp=re.findall(b"Login success", out) 
    assert exp==[b'Login success']
    system("mlsteam data mb bk/cifar10")
    system("mlsteam data cp -r /workspace/cifar10/* bk/cifar10")
    system("mlsteam data ls bk/cifar10")

def teardown_module(module):
    """ teardown any state that was previously setup with a setup_module
    method.
    """
    system("mlsteam data rb bk/cifar10")

def test_data_mb():
    ret=system("mlsteam data mb test")
    assert ret==0


def test_data_cp():
    ret=system("mlsteam data cp /workspace/tests/fake_dataset/file1 bk/test")
    assert ret==0


def test_data_ls():
    ret=system("mlsteam data ls bk/test")
    assert ret==0


def test_data_rm():
    ret=system("mlsteam data rm bk/test/file1")
    assert ret==0

def test_data_rb():
    ret=system("mlsteam data rb test")
    assert ret==0

def test_project_create():
    ret=system("mlsteam project create test_project cifar10")
    assert ret==0

def test_job_submit():
    ret=system("PROJECT=test_project mlsteam job submit training --job-name job1 --package-path /workspace/example/cifar10_estimator -- python2 cifar10_main.py --data-dir /dataset --job-dir /jobs --train-steps=100")
    assert ret==0


def test_job_list():
    ret=system("mlsteam job list")
    assert ret==0


def test_project_list():
    ret=system("mlsteam project list")
    assert ret==0


def test_job_status():
    job_status=os.popen("mlsteam job list | awk '/job1/ {print $8}'").read()
    job_status=job_status.rstrip()
    while job_status != "Done":
        job_status=os.popen("mlsteam job list | awk '/job1/ {print $8}'").read()
        job_status=job_status.rstrip()
        print(job_status)
        if job_status == "Running" or job_status == "Waiting" or job_status == "Initializing":
            time.sleep(10)
        elif job_status == "Error" or job_status == "Stopped":
            raise Exception('Job status: '+job_status)
        elif job_status != "Done":
            raise Exception('Unexpected job status: '+job_status)


def test_job_delete():
    job_ids=os.popen("mlsteam job list | awk '/job1/ {print $2}'").read()
    print(job_ids)
    print(job_ids.split('\n'))
    for job_id in job_ids.split('\n'):
        if job_id=='': continue
        ret=system("mlsteam job delete --job-id "+job_id)
        assert ret==0


def test_project_delete():
    proj_id=os.popen("mlsteam project list | awk '/test_project/ {print $2}'").read()
    print(proj_id)
    ret=system("mlsteam project delete --id "+proj_id)
    assert ret==0



import os
import json
import base64
from mlsteam import envs
from mlsteam.exceptions import (
    MLSteamMissingApiTokenException,
    MLSteamInvalidApiTokenException,
)

class Credential(object):
    def __init__(self, token):
        if token is None:
            token = os.getenv(envs.API_TOKEN_ENV)
        if token is None:
            raise MLSteamMissingApiTokenException()
        token_dict = self.api_token_to_dict(token)
        if "api_address" not in token_dict:
            raise MLSteamInvalidApiTokenException()
        self._api_address = token_dict["api_address"]
        self._api_token = token

    def api_token_to_dict(self, api_token):
        try:
            tokend = {}
            tokens = api_token.split('.')
            if len(tokens) != 3:
                raise MLSteamInvalidApiTokenException()
            tokend = self.token_decode(tokens[0])
            tokend.update(self.token_decode(tokens[1]))
            return tokend
        except Exception:
            raise MLSteamInvalidApiTokenException()

    def token_decode(self, token):
        try:
            tokenb = token.encode() + b'=' * (-len(token) % 4)
            return json.loads(base64.b64decode(tokenb).decode('utf-8'))
        except Exception:
            raise MLSteamInvalidApiTokenException()

    @property
    def api_token(self):
        return self._api_token

    @property
    def api_address(self):
        return self._api_address



import signal
import sys
import logging
import click
from . import version
from .model import model
from .job import job
from .project import project
from .auth import login
from .container import container
from .ds import data
from .checkpoint import checkpoint
from .service import service
from .work import work
from .info import info


def sigint_handler(signum, frame):
    logging.debug("signal {}, {}".format(signum, frame))
    click.echo()
    sys.exit()


signal.signal(signal.SIGINT, sigint_handler)
CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.version_option(version=version.__version__)
@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    pass

cli.add_command(login)
cli.add_command(data)
cli.add_command(model)
#cli.add_command(train)
cli.add_command(job)
cli.add_command(project)
cli.add_command(container)
cli.add_command(checkpoint)
cli.add_command(service)
cli.add_command(work)
cli.add_command(info)


if __name__ == '__main__':
    cli()



import os
import re
import time
from datetime import datetime, timedelta
import json
import logging
import getpass
import requests

DATA_TRANSFER_REMOTE=True


DEFAULT_TIMEOUT_S = 100
CONFIG_PATH = '.mlsteam'


class MyelindlApi(object):
    def __init__(self, address=None, username=None, data_port=9000):
        self.base_url = "http://{}/api".format(address) or 'http://localhost/api'
        self.address = address or 'localhost'
        if address and address.startswith('http'):
            raise MyelindlApiError('address has to be IP:Port format.')
        self.host = self.address.split(':')[0]
        self.username = username or getpass.getuser()
        self.config_path = os.path.join(os.getenv('HOME'), '.mlsteam')
        self.key_path = os.path.join(self.config_path, 'key')
        self.cred_file = os.path.join(self.config_path, 'cred')
        self.data_port = data_port
        self.access_token = None
        self.refresh_token = None
        self.access_token_acquire_time = None
        self.refresh_token_acquire_time = None

        if DATA_TRANSFER_REMOTE:
            if address is None or username is None:
                self.load_credential()

    def save_credential(self):
        if not os.path.exists(self.config_path):
            os.makedirs(self.config_path)
        if not self.address or not self.username:
            raise MyelindlApiError('Wrong input')

        with open(self.cred_file, encoding='utf-8', mode='w') as cred:
            json.dump({
                'address': self.address,
                'host': self.host,
                'username': self.username,
                'data_port': self.data_port,
                'access_token': self.access_token,
                'refresh_token': self.refresh_token,
                'access_token_acquire_time': str(self.access_token_acquire_time),
                'refresh_token_acquire_time': str(self.refresh_token_acquire_time),
            }, cred)

    def load_credential(self):
        if not os.path.exists(self.cred_file):
            raise MyelindlApiError('Login First')

        with open(self.cred_file, encoding='utf-8') as cred:
            data = json.load(cred)

        if 'address' not in data or 'username' not in data:
            raise MyelindlApiError('Login First')

        self.host = data['host']
        self.address = data['address']
        self.host = self.address.split(':')[0]
        self.base_url = "http://{}/api".format(self.address)
        self.username = data['username']
        self.access_token = data['access_token']
        self.refresh_token = data['refresh_token']
        self.data_port = data['data_port']
        self.access_token_acquire_time = float(data['access_token_acquire_time'])
        self.refresh_token_acquire_time = float(data['refresh_token_acquire_time'])

    def is_access_token_expired(self):
        return datetime.now() > (datetime.fromtimestamp(self.access_token_acquire_time) + timedelta(minutes=30))

    def is_refresh_token_expired(self):
        return datetime.now() > (datetime.fromtimestamp(self.refresh_token_acquire_time) + timedelta(days=30))

    def clear_credential(self):
        if os.path.exists(self.cred_file):
            os.remove(self.cred_file)

    def login(self, password):
        self.request_token(password)
        self.save_credential()

    def request_token(self, password):
        auth_token = self._request('auth/login',
            method='post',
            data={'username': self.username, 'password': password},
            access_token=None
        )
        self.access_token = auth_token['access_token']
        self.refresh_token = auth_token['refresh_token']
        self.access_token_acquire_time = time.time()
        self.refresh_token_acquire_time = time.time()

    def renew_token(self):
        if self.is_refresh_token_expired():
            raise MyelindlApiError({'error', 'Token expired pleas login Again.'})
        auth_token = self._request('auth/refresh',
            method='post',
            access_token=self.refresh_token,
            check_token_expire=False,
        )
        self.access_token = auth_token['access_token']
        self.access_token_acquire_time = time.time()

    def version(self):
        return self._request('version',
            method='get',
        )

    ############################
    # Dataset API
    ############################
    def minio_api_key(self):
        return self._request('minio_key',
            method='get',
            access_token=self.access_token,
        )

    def bucket_add(self, name):
        return self._request('buckets',
            method='post',
            access_token=self.access_token,
            data={
                'name': name,
            }
        )

    def bucket_del(self, name):
        return self._request('buckets/{}'.format(name),
            method='delete',
            access_token=self.access_token,
        )

    def dataset_publish(self, _id, _type, name, description, data_dir):
        return self._request('datasets',
            method='post',
            access_token=self.access_token,
            data={
                'id': _id,
                'name': name,
                'type': _type,
                'description': description,
                'data_dir': data_dir,
            }
        )

    def dataset_unpublish(self, _id):
        return self._request('datasets/{}/publish'.format(_id),
            method='delete',
            access_token=self.access_token,
        )

    def dataset_list(self):
        return self._request('datasets',
            method='get',
            access_token=self.access_token,
        )

    def dataset_items(self, _id, _dir=''):
        return self._request('datasets/{}/items'.format(_id),
            method='get',
            access_token=self.access_token,
            data={'path': _dir}
        )

    def dataset_info(self, _id):
        return self._request('datasets/{}'.format(_id),
            method='get',
            access_token=self.access_token,
        )

    ############################
    # Model API
    ############################
    def process_model_tag(self, model_tag):

        user = self.username
        model = model_tag
        tag = 'latest'

        user_and_model = model.split('/')
        if len(user_and_model) >= 2:
            user = user_and_model[-2]
            model = user_and_model[-1]

        model_and_tag = model.split(':')
        if len(model_and_tag) >= 2:
            model = model_and_tag[-2]
            tag = model_and_tag[-1]

        return '{}/{}:{}'.format(user, model, tag)

    def model_list(self, offset=0, limit=None):
        return self._request('models',
            method='get',
            access_token=self.access_token,
            data={'offset':offset, 'limit':limit},
        )

    def model_versions(self, model_name, offset=0, limit=None):
        return self._request('models/{}'.format(model_name),
            method='get',
            access_token=self.access_token,
            data={'offset':offset, 'limit':limit},
        )

    def model_push(self, model_tag, data_dir, description='', _type='file'):
        model_tag = self.process_model_tag(model_tag)
        return self._request('models/{}'.format(model_tag),
            method='post',
            access_token=self.access_token,
            data={
                'model_dir': data_dir,
                'description': description,
                'type': _type,
            }
        )

    def model_delete(self, model_tag):
        model_tag = self.process_model_tag(model_tag)
        return self._request('models/{}'.format(model_tag),
            method='delete',
            access_token=self.access_token,
        )

    def model_pull(self, model_tag, output_dir=''):
        model_tag = self.process_model_tag(model_tag)
        return self._request('models/{}/blob'.format(model_tag),
            method='get',
            access_token=self.access_token,
            output_format='file',
            output_file_dir=output_dir,
        )

    def model_info(self, model_tag):
        model_tag = self.process_model_tag(model_tag)
        return self._request('models/{}'.format(model_tag),
            method='get',
            access_token=self.access_token,
        )

    ############################
    # Server API
    ############################
    def list_server(self):
        return self._request('servers',
            method='get',
            access_token=self.access_token,
        )

    def create_server(self, network):
        return self._request('servers',
            method='post',
            access_token=self.access_token,
            data={'network': network}
        )

    def stop_server(self, server_id):
        return self._request('servers/{}'.format(server_id),
            method='delete',
            access_token=self.access_token,
        )

    def server_inference(self, server_id , image_file, total):
        with open(image_file,'rb') as f:
            return self._request('servers/{}/inference'.format(server_id),
                method='post',
                access_token=self.access_token,
                data={'total':total},
                files={'image': f},
            )

    ###########################
    # Work
    ###########################
    def work_list(self):
        return self._request('works',
            method='get',
            access_token=self.access_token,
        )

    def work_create(self, container, dataset, num_gpu, port, user_args):
        return self._request('works',
            method='post',
            access_token=self.access_token,
            data={'container': container,
                  'num_gpu': num_gpu,
                  'dataset': dataset,
                  'port': port,
                  'user_args': user_args}
        )

    def work_delete(self, _id):
        return self._request('works/{}'.format(_id),
            method='delete',
            access_token=self.access_token,
        )

    def work_info(self, _id):
        return self._request('works/{}'.format(_id),
            method='get',
            access_token=self.access_token,
        )

    ###########################
    # Service
    ###########################
    def service_list(self):
        return self._request('services',
            method='get',
            access_token=self.access_token,
        )

    def service_create(self, _id):
        return self._request('services',
            method='post',
            access_token=self.access_token,
            data={'checkpoint': _id}
        )

    def service_delete(self, _id):
        return self._request('services/{}'.format(_id),
            method='delete',
            access_token=self.access_token,
        )

    ###########################
    # Checkpoint
    ###########################
    def checkpoint_list(self):
        return self._request('checkpoints',
            method='get',
            access_token=self.access_token,
        )

    def checkpoint_download(self, _id):
        return self._request('checkpoints/{}/download'.format(_id),
            method='get',
            access_token=self.access_token,
        )

    def checkpoint_delete(self, _id):
        self._request('checkpoints/{}'.format(_id),
            method='delete',
            access_token=self.access_token,
        )

    def checkpoint_info(self, _id):
        return self._request('checkpoints/{}'.format(_id),
            method='get',
            access_token=self.access_token,
        )

    ###########################
    # Projects
    ###########################
    def project_create(self, name, dataset):
        return self._request('projects',
            method='post',
            access_token=self.access_token,
            data={'name':name,
                  'dataset':dataset}
        )

    def project_delete(self, project_id):
        self._request('projects/{}'.format(project_id),
            method='delete',
            access_token=self.access_token,
        )

    def project_list(self):
        return self._request('projects',
            method='get',
            access_token=self.access_token,
        )

    def project_get_info(self, project_id):
        return self._request('projects/{}'.format(project_id),
            method='get',
            access_token=self.access_token,
        )
    ###########################
    # Jobs
    ###########################
    def job_create(self, project, image_tag, job_name,
                   pkg_path, parameters, num_gpu, user_args):
        return self._request('jobs',
            method='post',
            access_token=self.access_token,
            data={'username': self.username,
                'project': project,
                'image-tag': image_tag,
                'job-name': job_name,
                'pkg-path': pkg_path,
                'num-gpu': num_gpu,
                'user-args': user_args,
                'parameters': parameters,
            }
        )

    def job_clone(self, pkg_path, job_id):
        return self._request('jobs/{}/repoclone'.format(job_id),
            method='post',
            access_token=self.access_token,
            data={'repo-path': pkg_path}
        )

    def job_commit(self, job_id):
        return self._request('jobs/{}'.format(job_id),
            method='post',
            access_token=self.access_token,
        )

    def job_train(self, job_id):
        return self._request('jobs/{}/train'.format(job_id),
            method='post',
            access_token=self.access_token,
        )

    def job_list(self):
        return self._request('jobs',
            method='get',
            access_token=self.access_token,
        )

    def job_log(self, _id):
        return self._request('jobs/{}/log'.format(_id),
            method='get',
            output_format='plain',
            access_token=self.access_token,
        )

    def job_delete(self, _id):
        return self._request('jobs/{}'.format(_id),
            method='delete',
            access_token=self.access_token,
        )

    def job_abort(self, _id):
        return self._request('jobs/{}/abort'.format(_id),
            method='POST',
            access_token=self.access_token,
        )

    def job_download(self, _id):
        return self._request('jobs/{}/download'.format(_id),
            method='get',
            access_token=self.access_token,
            output_format='file',
        )

    ###########################
    # Container images
    ############################
    def image_list(self):
        return self._request('images',
            method='get',
            access_token=self.access_token,
        )

    def image_delete(self, _id):
        return self._request('images/{}'.format(_id),
            method='delete',
            access_token=self.access_token,
        )

    def image_pull(self, tag):
        return self._request('images',
            method='post',
            access_token=self.access_token,
            data={'tag': tag}
        )

    ###########################
    # Utils
    ############################
    def _request(self, path,
            method='get',
            data=None,
            files=None,
            output_format='json',
            output_file_dir='',
            timeout=DEFAULT_TIMEOUT_S,
            access_token=None,
            headers=None,
            check_token_expire=True,
        ):
        if headers is None:
            headers = {}
        url = '{0}/{1}'.format(self.base_url, path)
        func = getattr(requests, method)
        if access_token:
            if self.is_access_token_expired() and check_token_expire:
                logging.warning("token expire renew token")
                self.renew_token()
                self.save_credential()
                access_token = self.access_token
            headers.update({
                'Authorization': 'Bearer {}'.format(access_token)
            })

        response = func(
            url,
            data=data,
            files=files,
            stream=True if output_format == 'file' else False,
            timeout=timeout,
            headers=headers,
        )
        self._handle_error_msg(response, output_format)

        return self._process_output(response, output_format, output_file_dir)

    def _handle_error_msg(self, response, form):
        try:
            response.raise_for_status()
        except Exception as e :
            err_msg = ''
            if form == 'json':
                try:
                    result = json.loads(response.text)
                    if 'error' in result and result['error']:
                        err_msg = result['error']
                    elif 'msg' in result and result['msg']:
                        err_msg = result['msg']
                except ValueError:
                    err_msg = f'{e}, output: {response.text}'
            else:
                err_msg = str(e)
            raise MyelindlApiError({'error': f"{response.status_code}:{err_msg}"})

    def _process_output(self, response, form, output_file_dir=''):
        result = None
        if form == 'json':
            try:
                result = json.loads(response.text)
                if 'error' in result and result['error']:
                    raise MyelindlApiError(result)
            except ValueError:
                raise MyelindlApiError({
                    'error': f"API return value format error. output: {response.text}"
                })
        elif form == 'plan':
            result = response.text
        elif form == 'file':
            d = response.headers['content-disposition']
            fname = re.findall("filename=(.+)", d)[0]
            fname = os.path.join(output_file_dir, fname)
            if output_file_dir != '' and not os.path.exists(output_file_dir):
                os.makedirs(output_file_dir)
            with open(fname, 'wb') as f:
                for chunk in response.iter_content(chunk_size=8192):
                    if chunk:
                        f.write(chunk)
        else:
            raise MyelindlApiError({
                'error': f"Output format not supported {form}"
            })

        return result


class MyelindlApiError(Exception):
    def __init__(self, result):
        self.result = result
        if isinstance(result, dict):
            self.message = result['error']
        else:
            self.message = result
        Exception.__init__(self, self.message)



import os
import re
from subprocess import check_output
import click
from .api import MyelindlApi

def os_system_enable_raise(cmd):
    ret = os.system(cmd)
#    click.echo(click.style("System call return {}".format(ret), fg='green'))
    if ret != 0:
        raise Exception

## Buckets ##
@click.command(name="mb")
@click.argument('name', required=True)
def mk_bk(name):
    """make a bucket"""
    try:
        bk_name = None
        if name.startswith('bk/'):
            bk_name = name[3:]
        else:
            bk_name = name
        #register to server
        api = MyelindlApi()
        api.bucket_add(bk_name)
    except Exception as e:
        click.echo(click.style("Bucket created failed. {}".format(e), fg='red'))
        raise
    click.echo(click.style("Bucket `{}` created successfully.".format(name), fg='green'))


@click.command(name="rb")
@click.argument('name', required=True)
def rm_bk(name):
    """remove a bucket"""
    try:
        bk_name = name
        if name.startswith('bk/'):
            bk_name = name[3:]
        api = MyelindlApi()
        api.bucket_del(bk_name)
    except Exception as e:
        click.echo(click.style("Bucket delete failed. {}".format(e), fg='red'))
        raise
    click.echo(click.style("Bucket `{}` deleted successfully.".format(name), fg='green'))


## Objects ##
@click.command()
@click.argument('target', required=False, nargs=-1)
def ls(target):
    """list buckets and objects"""
    try:
        if target:
            args = ["{}".format(a) for a in target]
            objs = " ".join(args)
            out_b = check_output("mc ls {}".format(objs), shell=True)
            out = bytes.decode(out_b)
            for line in out.splitlines():
                if 'run-' not in line:
                    match = re.search(r"(\[.*\])\s+([^\s]+B) (.*)", line)
                    if match:
                        click.echo(click.style(match.group(1), fg='green')+'\t'+
                                   click.style(match.group(2), fg='yellow')+' '+
                                   click.style(match.group(3), fg='bright_cyan'))
        else:
            click.echo(click.style("Are you want to list bucket? try `steam data ls bk/`", fg='green'))
    except Exception as e:
        click.echo(click.style("ls failed. {}".format(e), fg='red'))
        raise


@click.command()
@click.argument('source', required=True, nargs=-1)
def cat(source):
    """display object contents"""
    try:
        args = [a for a in source]
        objs = " ".join(args)
        os_system_enable_raise("mc cat {}".format(objs))
    except Exception as e:
        click.echo(click.style("cat object failed. {}".format(e), fg='red'))
        raise


@click.command()
@click.option('-n', '--lines', default=10, help="print the first 'n' lines (default: 10)")
@click.argument('source', required=True, nargs=-1)
def head(lines, source):
    """display first 'n' lines of an object"""
    try:
        args = [a for a in source]
        objs = " ".join(args)
        os_system_enable_raise("mc head -n {} {}".format(lines, objs))
    except Exception as e:
        click.echo(click.style("cat object failed. {}".format(e), fg='red'))
        raise


@click.command()
@click.option('-r', '--recursive', is_flag=True, help="copy recursively")
@click.argument('source', required=True, nargs=-1)
@click.argument('target', required=True)
def cp(recursive, source, target):
    """copy objects"""
    try:
        args = [a for a in source]
        objs = " ".join(args)
        if recursive:
            os_system_enable_raise("mc cp --recursive {} {}".format(objs, target))
        else:
            os_system_enable_raise("mc cp {} {}".format(objs, target))
    except Exception as e:
        click.echo(click.style("copy failed. {}".format(e), fg='red'))
        raise


@click.command()
@click.option('-r', '--recursive', is_flag=True, help="remove recursively")
@click.argument('target', required=True, nargs=-1)
def rm(recursive, target):
    """remove objects"""
    try:
        args = [a for a in target]
        objs = " ".join(args)
        if recursive:
            os_system_enable_raise("mc rm --recursive --force {} ".format(objs))
        else:
            os_system_enable_raise("mc rm {}".format(objs))
    except Exception as e:
        click.echo(click.style("remove failed. {}".format(e), fg='red'))
        raise


@click.command()
@click.option('--overwrite', is_flag=True, help="overwrite object(s) on target")
@click.option('--remove', is_flag=True, help="remove extraneous object(s) on target")
@click.argument('source', required=True)
@click.argument('target', required=True)
def mirror(overwrite, remove, source, target):
    """synchronize object(s) to a remote site"""
    try:
        cmd = "mc mirror "
        if overwrite:
            cmd += "--overwrite "
        if remove:
            cmd += "--remove "
        cmd += "{} {}".format(source, target)
        os_system_enable_raise(cmd)
    except Exception as e:
        click.echo(click.style("mirror failed. {}".format(e), fg='red'))
        raise


@click.group(help='Groups of commands to manage datasets')
def data():
    pass

data.add_command(mk_bk)
data.add_command(rm_bk)
data.add_command(ls)
data.add_command(cat)
data.add_command(head)
data.add_command(cp)
data.add_command(rm)
data.add_command(mirror)



import click
import platform
import threading
import queue
from urllib.parse import urlparse
from pathlib import Path
from time import time
from bravado.requests_client import RequestsClient
from bravado.client import SwaggerClient
from mlsteam.api_clients.credential import Credential
from mlsteam.version import __version__
from mlsteam.exceptions import MLSteamInvalidProjectNameException
ROOT_PATH = ".mlsteam"
MAX_AGGREGATE_LOG = 3000


class ConsumerThread(threading.Thread):
    def __init__(self,
        lock: threading.RLock,
        cache: "DiskCache",
        apiclient: "ApiClient",
        project_uuid: str,
        track_bucket_name: str,
        sleep_time: int
    ):
        super().__init__(daemon=True)
        self._lock = lock
        self._sleep_time = sleep_time
        self._interrupted = False
        self._event = threading.Event()
        self._is_running = False
        self._cache = cache
        self._apiclient = apiclient
        self._puuid = project_uuid
        self._track_bucket_name = track_bucket_name
        click.echo("Initialized track background thread")

    def is_running(self):
        return self._is_running

    def disable_sleep(self):
        self._sleep_time = 0

    def interrupt(self):
        self._interrupted = True
        self.wake_up()

    def wake_up(self):
        self._event.set()

    def run(self):
        self._is_running = True
        try:
            while not self._interrupted:
                # print("consumer start")
                self.work()
                # print("consumer stop")
                if self._sleep_time > 0 and not self._interrupted:
                    # print("sleep")
                    self._event.wait(timeout=self._sleep_time)
                    # print("wake")
                    self._event.clear()
                    # sleep for self._sleep_time
        finally:
            self._is_running = False

    def work(self):
        # while True:
        try:
            with self._lock:
                self._cache.process(
                    self._apiclient,
                    self._track_bucket_name
                )
        except Exception as e:
            click.echo("Error in mlsteam client api: {}".format(e))


class DiskCache(object):
    def __init__(self, track_path, debug=False):
        from shutil import rmtree
        self._queue = queue.Queue()
        self._debug = debug
        self.track_path = Path(ROOT_PATH, track_path)
        if not self.track_path.exists():
            self.track_path.mkdir(parents=True)
        else:
            for path in self.track_path.glob("**/*"):
                if path.is_file():
                    path.unlink()
                elif path.is_dir():
                    rmtree(path)

    def queue_size(self):
        return self._queue.qsize()

    def assign(self, key, value):
        op = QueueOp('config', {key: f"{value}"})
        self._queue.put(op)

    def log(self, key, value):
        tm = time()
        op = QueueOp('log', {key: f"{tm}, {value}\n"})
        self._queue.put(op)

    def process(self, apiclient: "ApiClient", bucket_name: str):
        i = MAX_AGGREGATE_LOG
        log_aggregate = {}
        while (not self._queue.empty()) and (i > 0):
            i = i - 1
            op = self._queue.get()
            if op.type == "config":
                self._write_config(op.content)
                for (keypath, value) in op.content.items():
                    if isinstance(value, str):
                        value = value.encode('utf-8')
                    apiclient.put_file(
                        bucket_name=bucket_name,
                        obj_path=keypath,
                        obj=value
                    )
            elif op.type == "log":
                self._write_log(op.content)
                for (key, value) in op.content.items():
                    if isinstance(value, str):
                        value = value.encode('utf-8')
                    if key in log_aggregate:
                        log_aggregate[key] = log_aggregate[key] + value
                    else:
                        log_aggregate[key] = value
        if log_aggregate:
            for (keypath, value) in log_aggregate.items():
                apiclient.put_file(
                    bucket_name=bucket_name,
                    obj_path=keypath,
                    obj=value,
                    part_offset='-1'
                )
            if self._debug:
                click.echo("queue size: {}".format(self.queue_size()))

    def _write_config(self, content):
        for (key, value) in content.items():
            key_path = self.track_path.joinpath(key)
            if not key_path.parent.exists():
                key_path.parent.mkdir(parents=True)
            with key_path.open('w') as f:
                if isinstance(value, str):
                    f.write(value.rstrip()+"\n")
                else:
                    f.write(f"{value}\n")

    def _write_log(self, content):
        for (key, value) in content.items():
            key_path = self.track_path.joinpath(f"{key}.log")
            if not key_path.parent.exists():
                key_path.parent.mkdir(parents=True)
            tm = time()
            with key_path.open('a') as f:
                f.write(f"{tm}, {value}\n")


class QueueOp(object):
    def __init__(self, optype, content):
        self._optype = optype
        self._content = content

    @property
    def type(self):
        return self._optype

    @property
    def content(self):
        return self._content


class ApiClient(object):
    def __init__(self, api_token=None):
        self.credential = Credential(api_token)
        self.http_client = create_http_client()
        self.http_client.set_api_key(
            host=urlparse(self.credential.api_address).netloc,
            api_key=f"Bearer {self.credential.api_token}",
            param_name="Authorization",
            param_in="header",
        )
        self.swagger_client = SwaggerClient.from_url(
            f"{self.credential.api_address}/api/v2/swagger.json",
            config=dict(
                validate_swagger_spec=False,
                validate_requests=False,
                validate_response=False
            ),
            http_client=self.http_client,
        )

    def get_project(self, name):
        result = self.swagger_client.project.listProject(
            name=name
        ).result()
        if result:
            project = result[0]
            if project:
                click.echo("Verified project from server, get project uuid {}".format(project['uuid']))
                return project['uuid']
        raise MLSteamInvalidProjectNameException()

    def create_track(self, project_uuid):
        result = self.swagger_client.track.createTrack(
            puuid=project_uuid
        ).result()
        click.echo("Create new track '{}' under project".format(result['name']))
        return result

    def get_track(self, project_uuid, track_id):
        result = self.swagger_client.track.getTrack(
            puuid=project_uuid,
            tid=track_id
        ).result()
        click.echo("Get track '{}' under project".format(result['name']))
        return result

    def put_file(self, bucket_name: str, obj_path: str, obj: bytes, part_offset: int = None):
        result = self.swagger_client.object.putObject(
            bucket_name=bucket_name,
            obj_path=obj_path,
            obj=obj,
            part_offset=part_offset,
            part_size=len(obj)
        ).result()
        return result

    def add_tags(self, project_uuid, track_id, tags: list):
        return self.swagger_client.track.addTagTrack(
            puuid=project_uuid,
            tid=track_id,
            tags=tags
        )


def create_http_client():
    http_client = RequestsClient()
    user_agent = (
        "mlsteam-client/{lib_version} ({system}, python {python_version})".format(
            lib_version=__version__,
            system=platform.platform(),
            python_version=platform.python_version(),
        )
    )
    http_client.session.headers.update({"User-Agent": user_agent})
    return http_client



import os
import time


class PrettyFloat(float):
    def __repr__(self):
        return '%.2f' % self

def pretty_floats(obj):
    if isinstance(obj, float):
        return PrettyFloat(obj)
    elif isinstance(obj, dict):
        return dict((k, pretty_floats(float(v))) for k, v in obj.items())
    return obj


def write(metrics):
    if not isinstance(metrics, dict):
        raise ValueError('input is not a dictionary!')
    _metrics = pretty_floats(metrics)
    log_file = os.getenv('METRICS_PATH', None)
    if log_file:
        t0=time.time()
        chunk="{}, {}\n".format(t0, _metrics)
        with open(log_file, encoding="utf-8", mode="a") as f:
            f.write(chunk)



import os.path
from setuptools import find_packages, setup
# from mlsteam.version import __version__
__version__ = "0.4.8"

with open(
    os.path.join(os.path.abspath(os.path.dirname(__file__)), "README.md"),
    encoding="utf-8",
) as handler:
    long_description = handler.read()


def main():
    root_dir = os.path.dirname(__file__)
    # Get requirements
    requirements = []
    with open(os.path.join(root_dir, 'requirements.txt'), 'r') as infile:
        for line in infile:
            line = line.strip()
            if line and not line[0] == '#':  # ignore comments
                requirements.append(line)
        setup(
            name='mlsteam-client',
            python_requires=">=3.6.0",
            version=__version__,
            description="MLSteam Client",
            author="MyelinTek inc.",
            author_email="simon@myelintek.com",
            url="https://myelintek.com",
            long_description=long_description,
            long_description_content_type="text/markdown",
            license='MIT',
            keywords=['Deep Learning Tracking'],
            packages=find_packages(),
            use_2to3=True,
            include_package_data=True,
            zip_safe=False,
            install_requires=requirements,
            # scripts=['bin/mc'],
            data_files=[('bin', ['bin/mc'])],
            entry_points={
                'console_scripts': ['mlsteam=mlsteam.cli:cli'],
            },
            classifiers=[
                "Development Status :: 3 - Alpha",
                "Environment :: Console",
                "Intended Audience :: Developers",
                "Intended Audience :: Science/Research",
                "License :: OSI Approved :: MIT License",
                "Natural Language :: English",
                "Operating System :: POSIX :: Linux",
                "Programming Language :: Python :: 3",
                "Programming Language :: Python :: 3.6",
                "Programming Language :: Python :: 3.7",
                "Programming Language :: Python :: 3.8",
                "Programming Language :: Python :: 3.9",
                "Topic :: Software Development :: Libraries :: Python Modules",
                "Programming Language :: Python :: Implementation :: CPython",
                "Topic :: Scientific/Engineering :: Artificial Intelligence"
            ]
        )


if __name__ == "__main__":
    main()



from mlsteam import envs


class MLSteamException(Exception):
    def __hash__(self):
        return hash((super().__hash__(), str(self)))


class MLSteamMissingApiTokenException(MLSteamException):
    def __init__(self):
        message = "API token {env_api_token} is missing"
        super().__init__(
            message.format(env_api_token=envs.API_TOKEN_ENV)
        )


class MLSteamInvalidApiTokenException(MLSteamException):
    def __init__(self):
        message = "API token is invalid, make sure your API token is correct."
        super().__init__(message)


# Project
class MLSteamMissingProjectNameException(MLSteamException):
    def __init__(self):
        message = "Project name {env_project_name} is missing"
        super().__init__(
            message.format(env_project_name=envs.PROJECT_ENV)
        )


class MLSteamInvalidProjectNameException(MLSteamException):
    def __init__(self):
        message = "Project name {env_project_name} is invalid"
        super().__init__(
            message.format(env_project_name=envs.PROJECT_ENV)
        )



import json
from datetime import datetime
import click
from .api import MyelindlApi, MyelindlApiError


@click.command()
@click.argument('name', required=True)
@click.argument('dataset', required=True)
def create(name, dataset):
    try:
        if dataset.startswith('bk/'):
            click.echo("no need 'bk/' prefix for dataset.")
            return
        api = MyelindlApi()
        result = api.project_create(name, dataset)

        click.echo('{}'.format(result['id']))
    except MyelindlApiError as e:
        click.echo("new project failed, {}".format(e))
        raise


@click.command('list')
@click.option('--json', 'is_json', default=False, help="return json format output")
def do_list(is_json):
    try:
        api = MyelindlApi()
        result = api.project_list()
        longest = 10
        if result:
            longest = max(len(p['name']) for p in result)
        if longest < 10:
            longest = 10
        if is_json:
            click.echo(json.dumps(result, indent=2, sort_keys=True))
            return
        template = '| {:>8} | {:>%d} | {:>8} | {:>21} |'% longest
        header = template.format('id', 'name', 'user', 'create time')
        click.echo('=' * len(header))
        click.echo(header)
        click.echo('=' * len(header))
        for proj in result:
            line = template.format(proj['id'],
                                   proj['name'],
                                   proj['username'],
                                   datetime.fromtimestamp(proj['create_time']).strftime("%Y %b %d, %H:%M:%S"))
            click.echo(line)
        click.echo('=' * len(header))
    except MyelindlApiError as e:
        click.echo("list project failed, {}".format(e))
        raise


@click.command()
@click.option('--id', required=True, help="id of the project")
def delete(_id):
    try:
        api = MyelindlApi()
        api.project_delete(_id)
    except MyelindlApiError as e:
        click.echo("delete a project failed, {}".format(e))
        raise


@click.command()
@click.option('id', '--id', required=True, help='id')
def info(_id):
    try:
        api = MyelindlApi()
        result=api.project_get_info(_id)

        template = '| {:>11} | {:>40}|'
        header = template.format('Field', 'Value')
        click.echo('=' * len(header))
        click.echo(header)
        click.echo('=' * len(header))

        for k, v in result['info'].iteritems():
            line = template.format(k, v)
            click.echo(line)
        click.echo('='* len(header))
    except MyelindlApiError as e:
        click.echo("show project info failed, {}".format(e))
        raise


@click.group(help='Groups of commands to manage project')
def project():
    pass


project.add_command(create)
project.add_command(do_list)
project.add_command(delete)
project.add_command(info)



import os
import json
import subprocess
from datetime import datetime
import click
from mlsteam.api import MyelindlApi, MyelindlApiError


@click.command()
@click.option('--job-name', required=True, help='job name')
@click.option('--package-path', help='package path (for package jobs)')
@click.option('--image-tag', help='docker images name (for container jobs)')
@click.option('--parameters', help='job hyperparameters for parallel run or hpo')
@click.option('--num-gpu', type=click.INT, default=1, help='number of GPU (default: 1)')
@click.argument('user-args', nargs=-1, type=click.Path())
def training(job_name, package_path, image_tag, parameters, num_gpu, user_args):
    job_id = None
    project = os.getenv('PROJECT', None)
    if project is None:
        click.echo("environment variable PROJECT not defined!, use export PROJECT=<project name> before training.")
        return
    if package_path:
        package_path = os.path.expanduser(package_path)
        if not os.path.exists(package_path):
            if not (package_path.startswith("ssh") or package_path.startswith("http")):
                click.echo('--package-path: {} not exists!'.format(package_path))
                return
    if parameters:
        if not os.path.exists(parameters):
            click.echo('parameter file: {} not exists'.format(parameters))
        with open(parameters, encoding='utf-8') as pfile:
            parameters = pfile.read()
    try:
        api = MyelindlApi()
        args = [a for a in user_args]
        result = api.job_create(
            project=project,
            image_tag=image_tag,
            job_name=job_name,
            pkg_path=package_path,
            parameters=parameters,
            num_gpu=num_gpu,
            user_args=' '.join(args),
        )
        job_id = result['id']
        if package_path:
            if (package_path.startswith("ssh://") or
                package_path.startswith("http")):
                pass
            else:
                bk_path = os.path.join("bk", job_id)
                subprocess.call("mc cp --recursive {} {}".format(package_path, bk_path), shell=True)
        click.echo('Job id: {}'.format(job_id))
        result = api.job_train(job_id)
        click.echo('Job {}: {}'.format(job_name, result))
    except MyelindlApiError as e:
        if job_id:
            api.job_delete(job_id)
        click.echo("submit failed, %s"% str(e))
        raise


@click.command('list')
@click.option('--json', 'is_json', default=False, help="return json format output")
def do_list(is_json):
    try:
        api = MyelindlApi()
        result = api.job_list()
        longest = 10
        if result['jobs']:
            longest = max(len(j['name']) for j in result['jobs'])
        if longest < 10:
            longest = 10
        if is_json:
            click.echo(json.dumps(result, indent=2, sort_keys=True)+'\n')
            return
        template = '| {:>16} | {:>%d} | {:>10} | {:>8} | {:>21} | {:>10} |'% longest
        header = template.format('id', 'name', 'project', 'status', 'create time', 'user')
        click.echo('=' * len(header))
        click.echo(header)
        click.echo('=' * len(header))
        for inst in result['jobs']:
            line = template.format(inst['id'],
                                   inst['name'],
                                   inst['project'],
                                   inst['status_history'][-1][0],
                                   datetime.fromtimestamp(inst['status_history'][0][1]).strftime("%Y %b %d, %H:%M:%S"),
                                   inst['username'])
            click.echo(line)
        click.echo('=' * len(header))
    except Exception as e:
        click.echo("submit failed, {}".format(e))
        raise


@click.command()
@click.option('--job-id', required=True, help='job id')
def log(job_id):
    try:
        api = MyelindlApi()
        click.echo(api.job_log(job_id))
    except MyelindlApiError as e:
        click.echo("failed, {}".format(e))
        raise


@click.command()
@click.option('--job-id', required=True, help='job id')
def delete(job_id):
    try:
        api = MyelindlApi()
        result = api.job_delete(job_id)
        click.echo('Job {} deleted, {}'.format(job_id, result))
    except Exception as e:
        click.echo("failed, {}".format(e))
        raise


@click.command()
@click.option('--job-id', required=True, help='job id')
def abort(job_id):
    try:
        api = MyelindlApi()
        api.job_abort(job_id)
        click.echo('Job {} aborted '.format(job_id))
    except Exception as e:
        click.echo("failed, {}".format(e))
        raise


@click.command()
@click.option('--job-id', required=True, help='job id')
def download(job_id):
    try:
        api = MyelindlApi()
        api.job_download(job_id)
    except Exception as e:
        click.echo("failed, {}".format(e))
        raise


@click.group(help='Groups of commands to manage submit')
def submit():
    pass


submit.add_command(training)


@click.group(help='Groups of commands to manage job')
def job():
    pass


job.add_command(submit)
job.add_command(do_list)
job.add_command(log)
job.add_command(delete)
job.add_command(abort)
job.add_command(download)
